<?php

if (!defined('ABSPATH'))
    exit;


class AWCFE_Fields
{


    private static $_instance = null;

    public $_version;

    public $itemSchema = array(
        'required' => array( // common field that exists for all type of fields
            'type' => 'text',
            'label' => '',
            'custom' => false,
            'active' => true,
            'placeholder' => '',
            'show_in_email' => false,
            'show_in_order_page' => true,
            'class' => array(),
            'priority' => '',
            'elementId' => '',
            'col' => 6,
            'default' => '',
            'custom_class' => '',
        ),
        'optional' => array( // fields which wont be existing for all type of fields
            'name' => '',
            'required' => false,
            'bindingKey' => '',
            'description' => '',
            'maxlength' => '',
            'subtype' => '',
            'rows' => '',
            'options' => array(
                'value' => '',
                'label' => '',
                'selected' => false
            ),


        )
    );

    public function __construct()
    {

    }

    public function saveFields($fields)
    {

        $validation = true;
        // now we can actually save the data
        $allowed = array(
            'a' => array(// on allow a tags
                'href' => true, // and those anchors can only have href attribute
                'target' => true,
                'class' => true,// and those anchors can only have href attribute
                'style' => true
            ),
            'b' => array('style' => true, 'class' => true),
            'strong' => array('style' => true, 'class' => true),
            'i' => array('style' => true, 'class' => true),
            'img' => array('style' => true, 'class' => true, 'src' => true),
            'span' => array('style' => true, 'class' => true),
            'p' => array('style' => true, 'class' => true)
        );
        if ($fields && is_array($fields)) {
            foreach ($fields as $secKey => $section) {
                if (isset($section['fields'][0][0])) {  // check if rowXcol arrange ment
                    foreach ($section['fields'] as $row => $col) {
                        foreach ($col as $colIndex => $val) {

                            $filteredValue = [];
                            foreach (array_merge($this->itemSchema['required'], $this->itemSchema['optional']) as $k => $v) {
                                if (isset($val[$k])) {
                                    $submitedVal = $val[$k];
                                    if ($k === 'label' && $val['type'] === 'paragraph') {
                                        $filteredValue[$k] = wp_kses($submitedVal, $allowed);
                                    } else if ($k === 'options') {
                                        $options = array();
                                        foreach ($submitedVal as $option) {
                                            array_push($options,
                                                array(
                                                    'value' => wp_kses($option['value'], array()),
                                                    'label' => wp_kses($option['label'], array()),
                                                    'selected' => (isset($option['selected']) && $option['selected'] === true) ? true : false
                                                ));
                                        }
                                        $filteredValue[$k] = $options;
                                    } else if (is_bool($v)) {
                                        $filteredValue[$k] = ($submitedVal === true) ? true : false;
                                    } else if (is_string($v)) {
                                        $filteredValue[$k] = wp_kses($submitedVal, array());
                                    } else if (is_numeric($v)) {
                                        $filteredValue[$k] = $submitedVal;
                                    } else if (is_array($v)) {
                                        $options = array();
                                        foreach ($submitedVal as $option) {
                                            array_push($options, wp_kses($option, array()));
                                        }
                                        $filteredValue[$k] = $options;
                                    }
                                }


                            }
                            $fields[$secKey]['fields'][$row][$colIndex] = $filteredValue;
                        }


                    }
                } else {
                    $fields[$secKey]['fields'][0] = [];
                }
            }

        } else {
            $fields = [];
        }


        $my_post = array(
            'fields' => $fields
        );

        update_option(AWCFE_FIELDS_KEY, $my_post);
        return $fields;

    }

    /**
     *
     */
    public function getFields($defaultFields)
    {


        $customSections = get_option(AWCFE_FIELDS_KEY);
        if ($customSections && isset($customSections['fields'])) {
            $customSections = $customSections['fields'];
        } else {
            $customSections = [];
        }


        $customSections = $this->spreadRowsCols($customSections);

        foreach ($defaultFields as $section => $fields) {
            if (isset($customSections[$section])) {
                $this->syncWithDefault($customSections[$section], $fields);
            } else {

                $customSections[$section] = $fields;

            }

            // uasort($customSections[$section], 'wc_checkout_fields_uasort_comparison');
        }

        if (defined('REST_REQUEST') && REST_REQUEST) {
            $customSections = $this->fieldsToRowCol($customSections);
            return $customSections;

        } else {

            $customSections = array_map(
                function ($section) {
                    $newArr = $section;

                    foreach ($section as $key => $field) {
                        if (isset($field['active']) && $field['active'] === false) {
                            unset($newArr[$key]);
                        } else {
                            if (isset($field['custom_class'])) {
                                $custom_class = preg_split('/[\ \n\,]+/', $field['custom_class']);
                                $newArr[$key]['class'] = array_merge($newArr[$key]['class'], $custom_class);
                            }
                            if (isset($field['type'])) {
                                if ($field['type'] === 'tel') {
                                    $newArr[$key]['validate'] = array_merge(isset($newArr[$key]['validate'])?$newArr[$key]['validate']:[], ['phone']);
                                }
                                if ($field['type'] === 'email') {
                                    $newArr[$key]['validate'] = array_merge(isset($newArr[$key]['validate'])?$newArr[$key]['validate']:[], ['email']);
                                }
                                //
                            }
                            if (isset($field['options']) && is_array($field['options'])) {
                                $newArr[$key]['options'] = [];
                                foreach ($field['options'] as $option) {
                                    $newArr[$key]['options'][$option['value']] = $option['label'];
                                    if (isset($option['selected']) && $option['selected'] === true) {
                                        $newArr[$key]['default'] = $option['value'];
                                    }
                                }
                            }
                            if (isset($field['rows'])) {
                                $newArr[$key]['custom_attributes'] = array_merge(isset($newArr[$key]['custom_attributes'])?$newArr[$key]['custom_attributes']:[], ['rows'=>$field['rows']]);
                            }
                        }


                    }

                    return $newArr;

                },
                $customSections
            );


            return $customSections;
        }


    }

    public function spreadRowsCols($sections)
    {

        $sections = array_map(
            function ($section) {
                $newAr = array();
                if (isset($section['fields'][0][0])) {// check if rowXcol arrange ment
                    foreach ($section['fields'] as $row => $col) {
                        foreach ($col as $i => $v) {
                            // check if disabled and exclude
                            if (isset($v['name'])) {
                                $key = $v['name'];
                            } else if (isset($v['elementId'])) {
                                $key = $v['elementId'];
                            } else {
                                continue;
                            }
                            if (empty($key)) {
                                continue;
                            }
                            $index = $i;
                            $newAr[$key] = $v;
                            $newAr[$key]['class'] = $this->generateClass($index, $v);
                            if ($v['type'] === 'text') {
                                $newAr[$key]['type'] = $v['subtype'];
                            }

                        }

                    }
                }


                return $newAr;
            },
            $sections
        );

        return $sections;
    }

    public function generateClass($index, $field)
    {
        $classes = [
            'form-row-wide' => 6,
            'form-row-last' => 3,
            'form-row-first' => 3
        ];
        if ($field['col'] === 6) {
            $class = 'form-row-wide';
        } else {
            if ($index > 0) {
                $class = 'form-row-last';
            } else {
                $class = 'form-row-first';
            }
        }
        $classes = array();
        if (isset($field['class']) && is_array($field['class'])) {
            $classes = $field['class'];
        } else if (isset($field['class'])) {
            $classes = [$field['class']];
        }

        foreach ($classes as $c => $v) {
            unset($classes[$c]);
        }

        return array_merge($classes, [$class]);

    }

    public function syncWithDefault(&$customSection, $defaultSectionFields)
    {

        //check if this key is existing in $result
        foreach ($defaultSectionFields as $key => $val) {
            $hasField = false;

            foreach ($customSection as $item) {
                if (isset($item['bindingKey']) && $key === $item['bindingKey']) {
                    $hasField = true;
                    $customSection[$key] = array_merge($val, $customSection[$key]);// will overite the keys in $val with, and will append if any thing new
                    break;
                }
            }
            if (!$hasField) {
                $customSection[$key] = $val;//$this->newItemWooToCustom($val, $key);

            }
        }

        foreach ($customSection as $k => $val) {
            $hasField = false;
            if (isset($val['bindingKey']) && !empty($val['bindingKey'])) {
                foreach ($defaultSectionFields as $key => $item) {
                    if ($key === $val['bindingKey']) {
                        $hasField = true;
                        break;
                    }
                }
                if (!$hasField) {
                    unset($customSection[$k]);

                }
            }

        }


    }

    public function fieldsToRowCol($sections)
    {
        $newArray = array();
        foreach ($sections as $section => $fields) {
            $newArray[$section] = array(
                "extra" => [
                    'key' => $section,
                    'name' => $this->getSectionDefaultTitle($section),
                    'status' => 1,

                ],
                "fields" => []
            );
            uasort($fields, 'wc_checkout_fields_uasort_comparison');
            $row = 0;
            $col = 0;
            foreach ($fields as $key => $val) {
                $newItem = $this->newItemWooToApi($val, $key);
                $newArray[$section]['fields'][$row][] = $newItem;
                if ($newItem['col'] === 6 || $col > 0) {
                    $row++;
                    $col = 0;
                } else if ($newItem['col'] === 3) {
                    $col++;
                }

            }

        }
        return $newArray;
    }

    /**
     * @param $customSection
     * @param $defaultSectionFields
     */
//    public function syncWithDefault(&$customSection, $defaultSectionFields)
//    {
//
//        //check if this key is existing in $result
//        foreach ($defaultSectionFields as $key => $val) {
//            $hasField = false;
//
//            foreach ($customSection['fields'] as $row) {
//                foreach ($row as $item) {
//                    if (isset($item['bindingKey']) && $key === $item['bindingKey']) {
//                        $hasField = true;
//                        break;
//                    }
//                }
//                if ($hasField) {
//                    break;
//                }
//            }
//            if (!$hasField) {
//                $this->appendNewItem($customSection['fields'], $val, $key);
//
//            }
//        }
//
//        foreach ($customSection as $row => $col) {
//            foreach ($col as $k => $val) {
//                $hasField = false;
//                if (isset($val['bindingKey']) && !empty($val['bindingKey'])) {
//                    foreach ($defaultSectionFields as $key => $item) {
//                        if ($key === $val['bindingKey']) {
//                            $hasField = true;
//                            break;
//                        }
//                    }
//                    if (!$hasField) {
//                        unset($customSection['fields'][$row][$k]);
//
//                    }
//                }
//            }
//
//        }
//
//
//    }
    /**
     * @param $section
     * @return string|void
     */
    public function getSectionDefaultTitle($section)
    {
//        if ($section === 'billing') {
//            return __('Billing Fields', 'woocommerce');
//        } else if ($section === 'shipping') {
//            return __('Shpping Fields', 'woocommerce');
//        } else {
        return __(ucfirst($section) . ' Fields', 'aco-checkout-field');
//        }
    }

    /**
     *
     */
    public function newItemWooToApi($item, $key)
    {
        $args = wp_parse_args($item, $this->itemSchema['required']);
        $newField = $args;

        foreach ($this->itemSchema['optional'] as $name => $val) {
            if (isset($args[$name])) {
                $newField[$name] = $args[$name];
            }
        }

        $newField['name'] = $key;
        $newField['col'] = $this->colFromClass($args['class']);

        switch ($args['type']) {
            case 'text':
            case 'password':
            case 'datetime':
            case 'datetime-local':
            case 'date':
            case 'month':
            case 'time':
            case 'week':
            case 'number':
            case 'email':
            case 'url':
            case 'tel':
            case 'hidden':
                $newField['type'] = 'text';
                $newField['subtype'] = $args['type'];
                break;
            default:
                $newField['type'] = $args['type'];
        }

        if (!isset($args['custom']) || $args['custom'] === false) {
            $newField['bindingKey'] = $key;
            $newField['show_in_email'] = true;
            $newField['show_in_order_page'] = true;
        }

        if(isset($item['custom_attributes']['rows'])){
            $newField['rows'] = $item['custom_attributes']['rows'];
        }


        return $newField;

    }


    function colFromClass($class)
    {
        $classes = [
            'form-row-wide' => 6,
            'form-row-last' => 3,
            'form-row-first' => 3
        ];
        foreach ($classes as $key => $val) {
            if (is_array($class) && in_array($key, $class)) {
                return $val;
            } else if (is_string($class) && $class == $key) {
                return $val;
            }
        }
        return 6;
    }

    public function getDefaultFields()
    {
        if (WC()->customer == null) {
            wc()->frontend_includes();
            WC()->session = new WC_Session_Handler();
            WC()->session->init();
            WC()->customer = new WC_Customer(get_current_user_id(), true);
        }


        $checkout_fields = WC()->checkout()->get_checkout_fields();

        return $checkout_fields;
    }


}
