<?php

if (!defined('ABSPATH'))
    exit;

class AWCFE_Front_End
{


    private static $_instance = null;

    public $_version;

    /**
     * The token.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $_token;
    /**
     * The plugin assets URL.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $assets_url;
    /**
     * The main plugin file.
     * @var     string
     * @access  public
     * @since   1.0.0
     */
    public $file;

    function __construct($file = '', $version = '1.0.0')
    {
// Load frontend JS & CSS

        $this->_version = $version;
        $this->_token = AWCFE_TOKEN;

        /**
         * Check if WooCommerce is active
         * */
        if ($this->check_woocommerce_active()) {


            $this->file = $file;

            $this->assets_url = esc_url(trailingslashit(plugins_url('/assets/', $this->file)));


            add_filter('woocommerce_checkout_fields', array($this, 'get_checkout_fields'), 99999, 1);

            add_filter('woocommerce_get_country_locale_default', array($this, 'get_country_locale_default'), 10, 1);

            add_action('woocommerce_checkout_update_order_meta', array($this, 'update_order_meta'), 10, 2);

            add_action('woocommerce_form_field', array($this, 'woocommerce_form_field'), 10, 4);

            add_action('woocommerce_order_details_after_order_table', array($this, 'order_details_after_order_table'), 10, 1);
            add_action('woocommerce_email_after_order_table', array($this, 'email_after_order_table'), 10, 1);


        }
    }

    public function check_woocommerce_active()
    {
        if (in_array('woocommerce/woocommerce.php', apply_filters('active_plugins', get_option('active_plugins')))) {
            return true;
        }
        if (is_multisite()) {
            $plugins = get_site_option('active_sitewide_plugins');
            if (isset($plugins['woocommerce/woocommerce.php']))
                return true;
        }
        return false;
    }

    public static function instance($parent)
    {
        if (is_null(self::$_instance)) {
            self::$_instance = new self($parent);
        }
        return self::$_instance;
    }

    public function email_after_order_table($order)
    {
        $order_id = $order->get_id();
        $awcf_data = get_post_meta($order_id, AWCFE_ORDER_META_KEY, true);
        if ($awcf_data) {
            echo '<table>';
            foreach ($awcf_data as $section => $fields) {
                uasort($fields, 'wc_checkout_fields_uasort_comparison');
                $row_template = '<tr ><td>%1$s</td><td>%2$s</td></tr>';
                foreach ($fields as $key => $val) {
                    if (isset($val['show_in_email']) && $val['show_in_email'] === true) {
                        echo sprintf($row_template, $val['label'], $val['value']);
                    }
                }
            }
            echo '</table>';
        }

    }

    public function order_details_after_order_table($order)
    {
        $order_id = $order->get_id();
        $awcf_data = get_post_meta($order_id, AWCFE_ORDER_META_KEY, true);
        if ($awcf_data) {
            echo '<table>';
            foreach ($awcf_data as $section => $fields) {
                uasort($fields, 'wc_checkout_fields_uasort_comparison');
                $row_template = '<tr ><td>%1$s</td><td>%2$s</td></tr>';
                foreach ($fields as $key => $val) {
                    if (isset($val['show_in_order_page']) && $val['show_in_order_page'] === true) {
                        echo sprintf($row_template, $val['label'], $val['value']);
                    }
                }
            }
            echo '</table>';
        }


    }

    public function woocommerce_form_field($field, $key, $args, $value)
    {
        if ($args['type'] === 'paragraph') {
            $field .= '<p class="' . AWCFE_TOKEN . '_paragraph_field " >' . do_shortcode(nl2br($args['label'])) . '</p>';
            if (!empty($field)) {
                $field_html = '';

                $field_html .= $field;
                $container_class = esc_attr(implode(' ', $args['class']));
                $sort = $args['priority'] ? $args['priority'] : '';
                $field_container = '<div class="form-row %1$s"  data-priority="' . esc_attr($sort) . '">%2$s</div>';
                $field = sprintf($field_container, $container_class, $field_html);
            }
        }
        if ($args['type'] === 'header') {
            $field .= '<' . $args['subtype'] . ' class="' . AWCFE_TOKEN . '_paragraph_field " >' . do_shortcode(nl2br($args['label'])) . '</' . $args['subtype'] . '>';
            if (!empty($field)) {
                $field_html = '';

                $field_html .= $field;
                $container_class = esc_attr(implode(' ', $args['class']));
                $sort = $args['priority'] ? $args['priority'] : '';
                $field_container = '<div class="form-row %1$s"  data-priority="' . esc_attr($sort) . '">%2$s</div>';
                $field = sprintf($field_container, $container_class, $field_html);
            }
        }

        return $field;
    }


    public function update_order_meta($order_id, $postData)
    {
        $checkout_fields = WC()->checkout()->get_checkout_fields();
        $fieldSchema = [];
        foreach ($checkout_fields as $sekKey => $section) {
            $fieldSchema[$sekKey] = [];
            foreach ($section as $key => $field) {
                if (isset($field['custom']) && $field['custom'] && isset($postData[$key])) {
                    $value = wc_clean($postData[$key]);
                    $meta_id = false;
                    if ($value) {
                        $meta_id = update_post_meta($order_id, $key, $value);
                    }
                    if (!in_array($field['type'], ['paragraph', 'header'])
                        || (isset($field['show_in_email']) && $field['show_in_email'] === true)
                        || (isset($field['show_in_order_page']) && $field['show_in_order_page'] === true))
                        $fieldSchema[$sekKey][] = array(
                            'type' => $field['type'],
                            'meta_id' => $meta_id,
                            'name' => $field['name'],
                            'label' => (isset($field['label'])) ? (($field['label'] == '') ? AWCFE_EMPTY_LABEL : $field['label']) : AWCFE_EMPTY_LABEL,
                            'value' => $value,
                            'priority' => $field['priority'],
                            'col' => $field['col'],
                            'show_in_email' => isset($field['show_in_email']) ? $field['show_in_email'] : false,
                            'show_in_order_page' => isset($field['show_in_order_page']) ? $field['show_in_order_page'] : false,
                        );

                }
            }
        }
        if (!empty($fieldSchema)) {
            update_post_meta($order_id, AWCFE_ORDER_META_KEY, $fieldSchema);
        }
    }

    function get_country_locale_default($fields)
    {
        if (is_array($fields)) {

            foreach ($fields as $key => $val) {
                if (isset($val['priority'])) {
                    unset($fields[$key]['priority']);
                }

            }
        }
        return $fields;
    }

    /**
     * @param $defaultFields
     * @return array|mixed|void
     */
    public function get_checkout_fields($defaultFields)
    {
        $fields = new AWCFE_Fields();
        return $fields->getFields($defaultFields);
    }





// End enqueue_scripts ()
// End instance()
}
